#include "StdAfx.h"
#include "UpdaterDocument.h"
#include "XMLUtil.h"
#include <atlpath.h>
#include "HttpClient/httpclient.h"
#include "Binding.h"
#include "ScriptletContainerFactory.h"
#include "Application.h"
#include "CustomProgressBar.h"
#include "../../../common/ConfigXmlParser.h"
#include "UpdaterNotifyWindowQueue.h"
#include "LocalStorage/LocalStorage.h"
#include "CheckServiceUpgradeHelper.h"
#include "../../../common/drive.h"
#include <memory>
#include "TrayIconHandler.h"
#include "NaverAgent/NaverAgentUtil.h"
#include "PatchNaverTalk.h"
#include <strsafe.h>
#include <vector>

namespace tipupdater
{
const TCHAR CONFIG_XML_FILENAME[]				= _T("NtalksUpdater.xml");
const TCHAR CONFIG_CONTEXT_NAME[]				= _T("config");
const TCHAR CONFIG_FILE_NAME[]					= _T("NtalkbizUpgraderConfig");
const TCHAR CHECK_EVENT_TIME_CONTEXT_NAME[]		= _T("checkeventtime");

UpdaterDocument::UpdaterDocument():m_ConfigModel(this),m_CommonModel(this)
{
	m_bUpdateTab = TRUE;
	m_PendingService = NULL;
	m_bIsShowConfig = FALSE;
	m_bShowTooltipCloseMainViewFirstTime = FALSE;

// 	TCHAR statUrl[tipbase::MAXURL] = {0, };
// 	tipcommon::ConfigXmlParser::GetURL(GetConfigXmlName(), tipcommon::Element::StatUrl, statUrl, tipbase::MAXURL);
	m_NotifyWindowQueue = new UpdaterNotifyWindowQueue;
	m_bIsShowingNotifyWindow = FALSE;
	m_bInitListLayout = FALSE;
	InitializeCriticalSection(&m_criticalSection);
}

UpdaterDocument::~UpdaterDocument()
{	
	if (m_NotifyWindowQueue)
		delete m_NotifyWindowQueue;
	m_NotifyWindowQueue = NULL;
	DeleteCriticalSection(&m_criticalSection);
}

void UpdaterDocument::SetPendingService(TipServiceData* service) 
{ 
	m_PendingService = service; 
	/*
	if( NULL != service )
		TrayIconHandler::CreateInstacne()->SetUpdatingToolTip();
	else
		TrayIconHandler::CreateInstacne()->SetEmptyToolTip();
		*/
}

BOOL UpdaterDocument::Open()
{
	BOOL bSuccess = __super::Open();
	if (bSuccess)
	{
		LoadConfigModel();
		LoadCheckEventTime();
		CComVariant var = this->GetProperty(_T("config.autostart"));
		CComVariant var1 = this->GetProperty(_T("config.checkcorevalue"));

		this->AddDataContext(_T("updateinfo"));
		this->UpdateFreeCapacity();
		this->SetServicesVolume(0);
		this->ConfirmInstallStatus(false);

	}

	return bSuccess;
}
BOOL UpdaterDocument::CheckInstallInfo()
{
	TipServiceData* service = NULL;

	AgentDLLWrapper* agentDllWrapper = NULL;
	NAVERAPP_INSTALL_STATUS InstallStatus;
	int numOfNotInstalled = 0;
	int numOfNotUpdated = 0;
	BOOL bChanged = FALSE;
	InstallInfo::type oldInfo = InstallInfo::NOTINSTALLED;

	IDataContext* serviceList = GetChild(L"servicelist");
	std::vector<TipServiceData*> notInstalledList;
	std::vector<TipServiceData*> updatableList;
	std::vector<TipServiceData*> recentList;
	
	int count = GetServiceCount();
	for (int i = 0; i < count; i++)
	{
		service = GetService(i);
		IDataContext* serviceDataContext = serviceList->GetChild(i);
		oldInfo = service->GetInstallInfo();

		// ⺼ġ    üũŲ.
		if(service->IsDefaultInstall())
		{
			service->SetInstallSelect(TRUE);
		}

		agentDllWrapper = service->GetAgentDll();
		if (agentDllWrapper)
		{
			agentDllWrapper->GetInstalledStatus(InstallStatus);
			if (oldInfo != InstallStatus)
				bChanged = TRUE;

			switch(InstallStatus)
			{
			case NotInstalled:
				numOfNotInstalled++;
				if (oldInfo != InstallInfo::NOTINSTALLED)
					service->SetInstallInfo(InstallInfo::NOTINSTALLED);
				notInstalledList.push_back(service);
				break;
			case OldVersion:
				numOfNotUpdated++;
				if (oldInfo != InstallInfo::OLDVERSION)
					service->SetInstallInfo(InstallInfo::OLDVERSION);
				updatableList.push_back(service);
				break;
			case LatestVersion:
				//numOfNotUpdated++;
				if (oldInfo != InstallInfo::RECENTVERSION)
					service->SetInstallInfo(InstallInfo::RECENTVERSION);
				recentList.push_back(service);
				break;
			}
			agentDllWrapper = NULL;
		}
		else
		{
			numOfNotInstalled++;
			if (oldInfo != InstallInfo::NOTINSTALLED)
				service->SetInstallInfo(InstallInfo::NOTINSTALLED);
			notInstalledList.push_back(service);
		}
	}

	// ̼ġ, Ʈ, ġ  Ѵ.
	ATLASSERT(serviceList != NULL );
	serviceList->RemoveAll();
	ClearServiceData();

	CString name;
	int index = 0;

	// ʹ Ʈ  ׸  
	for (std::vector<TipServiceData*>::iterator it = updatableList.begin(); it != updatableList.end(); it++ )
	{
		name.Format(_T("%d"), index++);		
		AddServiceData(*it);
		serviceList->Add( name, (*it)->GetDataContext() );				
	}

	for (std::vector<TipServiceData*>::iterator it = notInstalledList.begin(); it != notInstalledList.end(); it++ )
	{
		name.Format(_T("%d"), index++);
		AddServiceData(*it);
		serviceList->Add( name, (*it)->GetDataContext() );
	}
	
	for (std::vector<TipServiceData*>::iterator it = recentList.begin(); it != recentList.end(); it++ )
	{
		name.Format(_T("%d"), index++);
		AddServiceData(*it);
		serviceList->Add( name, (*it)->GetDataContext() );
	}

	// ListView [ġ ] [Ϸ] ư visible binding Ѵ.
	int nUpdatableCount = notInstalledList.size() + updatableList.size();
	if( nUpdatableCount > 0 )
	{
		SetProperty(L"listview_isupdatable", L"true");
	}
	else
	{
		SetProperty(L"listview_isupdatable", L"false");
	}

	// ⺻   Ѵ.
	int default_selected = -1;
	count = this->GetCountOfServiceData(L"servicelist");
	for (int i = 0; i < count; i++)
	{
		service = this->GetService(i);
		if( service->GetInstallInfo() != InstallInfo::RECENTVERSION )
		{
			if( service->GetInstallSelect())
			{
				default_selected = i;
				break;
			}
		}
	}

	SetDefaultListIndex(default_selected);	

	if (bChanged)
	{
		int numOfInstalled = GetServiceCount() - numOfNotInstalled;
		SetInstalledListCount(numOfInstalled);
		SetUpdateListCount(numOfNotUpdated);
		SetInstallListCount(numOfNotInstalled);
		return TRUE;
	}	
	return FALSE;
}

BOOL UpdaterDocument::UpdateInstallInfo(LPCTSTR guid)
{
	int numOfNotInstalled = GetInstallListCount();
	int numOfNotUpdated = GetUpdateListCount();
	
	BOOL bChangedInfo = FALSE;

	TipServiceData* serivceData = SearchService(guid);
	if (serivceData)
	{
		InstallInfo::type oldInfo = serivceData->GetInstallInfo();

		AgentDLLWrapper* agentDllWrapper = NULL;
		agentDllWrapper = serivceData->GetAgentDll();
		if ( NULL == agentDllWrapper )
		{
			return FALSE;
		}

		NAVERAPP_INSTALL_STATUS InstallStatus;
		agentDllWrapper->GetInstalledStatus(InstallStatus);

		InstallInfo::type newInfo;
		switch(InstallStatus)
		{
		case NotInstalled:
			newInfo = InstallInfo::NOTINSTALLED;
			break;
		case OldVersion:
			newInfo = InstallInfo::OLDVERSION;
			break;
		case LatestVersion:
			newInfo = InstallInfo::RECENTVERSION;
			break;
		}
		if (newInfo == oldInfo)
			return bChangedInfo;

		if ((oldInfo == InstallInfo::OLDVERSION || oldInfo == InstallInfo::RECENTVERSION)
			&& newInfo == InstallInfo::NOTINSTALLED)
		{
			bChangedInfo = TRUE;
			numOfNotInstalled ++;
			//   ̾ٸ Ʈ Ʈ 
			if (oldInfo == InstallInfo::OLDVERSION)
				numOfNotUpdated--;
		}
		else if ((newInfo == InstallInfo::OLDVERSION || newInfo == InstallInfo::RECENTVERSION)
			&& (oldInfo == InstallInfo::NOTINSTALLED))
		{
			bChangedInfo = TRUE;
			numOfNotInstalled --;
			//   ̶ Ʈ Ʈ 
			if (newInfo == InstallInfo::OLDVERSION)
				numOfNotUpdated ++;
		}
		else if (( newInfo == InstallInfo::RECENTVERSION)
			&& (oldInfo == InstallInfo::OLDVERSION))
		{
			bChangedInfo = TRUE;
			//   ̾ٸ Ʈ Ʈ 
			if (oldInfo == InstallInfo::OLDVERSION)
				numOfNotUpdated --;
		}
		serivceData->SetInstallInfo(newInfo);

		int numOfInstalled = GetServiceCount() - numOfNotInstalled;
		SetInstalledListCount(numOfInstalled);
		SetUpdateListCount(numOfNotUpdated);
		SetInstallListCount(numOfNotInstalled);
		return bChangedInfo;
	}

	return bChangedInfo;
}
CTrayIcon* UpdaterDocument::GetTrayIcon()
{
	return TrayIconHandler::CreateInstacne()->GetTrayIcon();
}
BOOL UpdaterDocument::Reload()
{
	return CheckInstallInfo();
}

//////////////////////////////////////////////////////////////////////////
//UpdaterConfigModel
//////////////////////////////////////////////////////////////////////////
UpdaterConfigModel::UpdaterConfigModel(TipDocument* doc):m_Document(doc)
{
}

class ConfigLoader
{
public:
	void SetDefaultOption(UpdaterConfigModel *config, tipcommon::TipDocument *document )
	{		
		if( NULL == config || NULL == document )
			return;

		document->AddDataContext(_T("config"));
		config->SetUseNotifyUpdate	(TRUE);
		config->SetUseTrayIcon		(TRUE);
		config->SetUseCheckCoreValue(TRUE);
	}

	BOOL Load(UpdaterConfigModel *config, IDataContext *dataContext)
	{
		ATLASSERT( config );
		if( NULL == config)
			return FALSE;

		TCHAR config_file_position[1024];
		if( FALSE == Drive::GetCurrentUserLocalStorageFolderFileName( config_file_position, 1024, CONFIG_FILE_NAME) )
		{
			return FALSE;
		}

		std::auto_ptr<tipbase::LocalStorageManager> storage(new tipbase::LocalStorageManager(new tipbase::FileTool((TCHAR *)config_file_position)));

		int datasize = storage->Load( CONFIG_CONTEXT_NAME, NULL, 0, 0 );
		if( 0 == datasize )
		{
			return FALSE;
		}

		char *data = new char[datasize+2];
		if( NULL == data )
		{
			return FALSE;
		}

		storage->Load( CONFIG_CONTEXT_NAME, NULL, data, datasize );
		data[datasize] = 0;

		TiXmlDocument doc;
		doc.Parse(data);

		TiXmlElement* element =  doc.FirstChildElement();
		XMLDataProvider xmlProvider( dataContext );
		xmlProvider.Load(element);

		// [:2008-12-18] Memory Leak  : delete 
		delete [] data;

		return TRUE;
	}
};

class ConfigSaver
{
public:
	void MakeXML(CString &strXML, CString elementXML)
	{
		strXML += _T("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n");
		strXML += _T("<root>\r\n");
		strXML += elementXML;
		strXML += _T("</root>");
	}

	CString BoolToString(BOOL value)
	{
		if( value )
			return _T("true");

		return _T("false");
	}

	CString MakeConfigXML(	
		BOOL bNotifyUpdate, 
		BOOL bShowTray, 
		BOOL bCheckCoreValue,
		CString strInstallPath)
	{
		CString strElement;
		strElement = _T("<config>");
		strElement += _T("<notifyupdate>");
		strElement += BoolToString(bNotifyUpdate);
		strElement += _T("</notifyupdate>");
		strElement += _T("<usetrayicon>");
		strElement += BoolToString(bShowTray);
		strElement += _T("</usetrayicon>");
		strElement += _T("<checkcorevalue>");
		strElement += BoolToString(bCheckCoreValue);
		strElement += _T("</checkcorevalue>");
		strElement += _T("<installpath><![CDATA[");
		strElement += strInstallPath;
		strElement += _T("]]></installpath>");
		strElement += _T("</config>");

		CString strXML;
		MakeXML(strXML, strElement);
		USES_CONVERSION;

		return strXML;
	}

	BOOL Save( UpdaterConfigModel *config )
	{
		ATLASSERT( config );
		if( !config )
			return FALSE;

		TCHAR config_file_position[1024];
		Drive::GetCurrentUserLocalStorageFolderFileName( config_file_position, 1024, CONFIG_FILE_NAME);
		std::auto_ptr<tipbase::LocalStorageManager> storage(new tipbase::LocalStorageManager(new tipbase::FileTool((TCHAR *)config_file_position)));

		BOOL bNotifyUpdate, bShowTray,bCheckCoreValue;
		CString strPath;
		GetConfigData( config, bNotifyUpdate, bShowTray, bCheckCoreValue, strPath );

		CString xml = MakeConfigXML( bNotifyUpdate, bShowTray, bCheckCoreValue, strPath );
		CStringA xmlAscii = XMLUtil::Encode_utf8(xml);
		//CT2CA xmlAscii(xml);

		if( 0 == storage->Save( CONFIG_CONTEXT_NAME, NULL, (const char *)xmlAscii, strlen(xmlAscii) ) )
			return FALSE;

		return TRUE;
	}

	void GetConfigData( UpdaterConfigModel *config, 
		BOOL& bNotifyUpdate, 
		BOOL& bShowTray, 
		BOOL& bCheckCoreValue,
		CString& strInstallPath)
	{
		bNotifyUpdate	= config->IsUseNotifyUpdate();
		bShowTray		= config->IsUseTrayIcon();
		bCheckCoreValue	= config->IsUseCheckCoreValue();
		strInstallPath = config->GetInstallPath();
	}
};

class CheckEventTimeLoader
{
public:
	void ParseXML( const char *data, UpdaterDocument *doc )
	{
		TiXmlDocument xmlDoc;
		xmlDoc.Parse(data);
		
		TiXmlElement* element =  xmlDoc.FirstChildElement();

		SimpleMapDataContext dataContext;
		XMLDataProvider xmlData(&dataContext);
		xmlData.Load( element );

		IDataContext* installlistDataContext = dataContext.GetChild(_T("servicelist"));
		if (installlistDataContext != NULL)
		{
			int count = installlistDataContext->GetCount();
			for(int index = 0; index < count; index++)
			{
				IDataContext* serviceDataContext = installlistDataContext->GetChild(index);
				if (serviceDataContext != NULL)
				{
					CComVariant var = serviceDataContext->GetProperty(_T("guid"));
					CString guid = var;
					var = serviceDataContext->GetProperty(_T("chekc_event_time"));
					CString eventtime = var;
					
					if( eventtime == _T("") )
						continue;

					TipServiceData *service = doc->SearchService(guid);
					if( NULL == service )
						continue;

					CheckServiceUpgradeHelper helper(service);
					helper.SetServiceCheckTime(eventtime);
				}			
			}
		}	
		
	}

	void Load( UpdaterDocument *doc )
	{
		ATLASSERT( doc );
		if( NULL == doc)
			return;


		TCHAR config_file_position[1024];
		Drive::GetCurrentUserLocalStorageFolderFileName( config_file_position, 1024, CONFIG_FILE_NAME);
		std::auto_ptr<tipbase::LocalStorageManager> storage(new tipbase::LocalStorageManager(new tipbase::FileTool((TCHAR *)config_file_position)));

		int datasize = storage->Load( CHECK_EVENT_TIME_CONTEXT_NAME, NULL, 0, 0 );
		if( 0 == datasize )
		{
			return;
		}

		char *data = new char[datasize+2];
		if( NULL == data )
			return;

		storage->Load( CHECK_EVENT_TIME_CONTEXT_NAME, NULL, data, datasize );
		data[datasize] = 0;

		ParseXML(data, doc);

		// [:2008-12-18] Memory Leak  : delete 
		delete [] data;
	}
};

class CheckEventTimeSaver
{
public:
	void MakeXML(CString &strXML, CString elementXML)
	{
		strXML += _T("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n");
		strXML += _T("<root>\r\n");
		strXML += elementXML;
		strXML += _T("</root>");
	}

	CString MakeCheckEventTimeXML( UpdaterDocument *doc )
	{
		CString strElement = _T("");
		int count = doc->GetServiceCount();
		TipServiceData *service = NULL;
		for (int i = 0; i < count; i++)
		{
			service = doc->GetService(i);
			if( NULL == service )
				continue;
		
			CheckServiceUpgradeHelper helper(service);
			
			CString xml;
			CString id = helper.GetGuid();
			CString checkEventTime = helper.GetServiceCheckTime();
			if( checkEventTime == _T("") )
				continue;

			xml.Format( _T("<service guid=\"%s\"><chekc_event_time>%s</chekc_event_time></service>\r\n"), id, checkEventTime );
			strElement += xml;
		}

		CString strXML;
		MakeXML(strXML, strElement);
		USES_CONVERSION;

		return strXML;
	}

	BOOL Save( UpdaterDocument *doc )
	{
		TCHAR config_file_position[1024];
		Drive::GetCurrentUserLocalStorageFolderFileName( config_file_position, 1024, CONFIG_FILE_NAME);
		std::auto_ptr<tipbase::LocalStorageManager> storage(new tipbase::LocalStorageManager(new tipbase::FileTool((TCHAR *)config_file_position)));
		
		CString xml = MakeCheckEventTimeXML( doc );
		CT2CA xmlAscii(xml);

		if( 0 == storage->Save( CHECK_EVENT_TIME_CONTEXT_NAME, NULL, (const char *)xmlAscii, strlen(xmlAscii) ) )
			return FALSE;

		return TRUE;
	}
};

void UpdaterConfigModel::Init()
{
	ATLASSERT(m_Document);
	ConfigLoader loader;
	if( FALSE == loader.Load(this, m_Document->GetRoot()) )
	{
		loader.SetDefaultOption( this, m_Document );
	}
	m_Document->AddDataContext(_T("common"));	
}

BOOL UpdaterConfigModel::Save()
{
	ConfigSaver saver;
	return saver.Save(this);
}

BOOL UpdaterConfigModel::IsUseCheckCoreValue()
{
	return (NaverAgent_IsRunning() && NaverAgent_IsEnabled());
}

void UpdaterConfigModel::SetUseCheckCoreValue( BOOL value )
{
	if ( IsUseCheckCoreValue() != value )
	{
		if ( NaverAgent_KeepDS( value ) )
		{
			if ( value )
			{
				tipbase::GenericHTTPClient httpNClicks;
				httpNClicks.SetNclicks();
				httpNClicks.Request(NULL, NCLICK_AGENT_TOOLS_YES);
			}
		}
	}
}

void UpdaterDocument::SetInstallPath( LPCTSTR path )
{
	BOOL bDefaultPath = FALSE;
	if (path == NULL)
		bDefaultPath = TRUE;
	else if (_tcslen(path) == 0)
		bDefaultPath = TRUE;

	if (bDefaultPath == TRUE)
	{
		// ġ  
		TCHAR defaultpath[tipbase::MAXPATH] = {0, };
		SHGetSpecialFolderPath(NULL, defaultpath, CSIDL_PROGRAM_FILES, FALSE);
		ATLPath::Append(defaultpath, Application::LoadString(_T("IDS_DEFAULT_PATHNAME")));
		SetProperty(_T("config.installpath"), defaultpath);
	}
	else
	{
		SetProperty(_T("config.installpath"), path);
		UpdateFreeCapacity();
	}	
}

CString UpdaterDocument::GetInstallPath()
{
	CComVariant v = GetProperty(_T("config.installpath"));
	if( v.vt != VT_BSTR )
	{
		SetInstallPath(NULL);
		v = GetProperty(_T("config.installpath"));
	}

	if( v.vt == VT_BSTR )
	{
		return CString (v.bstrVal);
	}
	return L"";
}

CString UpdaterConfigModel::GetInstallPath()
{
	UpdaterDocument* doc = dynamic_cast<UpdaterDocument*>(m_Document);
	CString str;
	if(doc)
	{
		str = doc->GetInstallPath();
	}
	return str;
}

BOOL UpdaterDocument::SaveConfigModel()
{
	return m_ConfigModel.Save();
}

void UpdaterDocument::LoadConfigModel()
{
//	AddDataContext(_T("config"));
	m_ConfigModel.Init();
}

void UpdaterDocument::LoadCheckEventTime()
{
	CheckEventTimeLoader loader;
	loader.Load(this);
}

BOOL UpdaterDocument::SaveCheckEventTime()
{
	CheckEventTimeSaver saver;
	return saver.Save(this);
}

BOOL UpdaterDocument::MakeToBeInstallList()
{
	BOOL bAdd = FALSE;
	IDataContext* installlistDataContext = this->GetChild(_T("installlist"));
	ATLASSERT(installlistDataContext);
	if (installlistDataContext == NULL) return FALSE;

	installlistDataContext->RemoveAll();
	TipServiceData* service = NULL;
	TipServiceData* default_service = NULL;
	int index = 0;
	int contextIndex = 0;

	std::vector<TipServiceData*>::iterator it = m_ServiceList.begin();
	for (; it != m_ServiceList.end(); it++, contextIndex++)
	{
		service = *it;
		// ֽ  ƴ 쿡 ߰Ѵ.
		if ( service->GetInstallSelect() && service->GetInstallInfo() != InstallInfo::RECENTVERSION)
		{
			service->SetInstallStatus(InstallStatus::INIT);
			installlistDataContext->Add(NULL, service->GetDataContext());
			bAdd = TRUE;
		}
	}
	return bAdd;
}

int UpdaterDocument::GetNumOfToBeInstall()
{
	return GetCountOfServiceData(_T("installlist"));
}

TipServiceData* UpdaterDocument::GetToBeInstall( int index )
{
	return GetServiceData(_T("installlist"), index);
}

float UpdaterDocument::GetFreeCapacity()
{
	UpdateFreeCapacity();			
	CComVariant var = GetProperty(_T("common.freevolume"));
	return var.fltVal;
}

float UpdaterDocument::GetNeedCapacity()
{
	CComVariant var = GetProperty(_T("common.volume_sum"));
	if( var.vt == VT_R4 )
	{
		return var.fltVal;
	}
	return 0;
}

void UpdaterDocument::UpdateFreeCapacity()
{
	ULONGLONG FreeByteAvailableToCaller;
	CString pPath = GetInstallPath();
	TCHAR drivePath[tipbase::MAXPATH] = {0, };
	_tcsncpy_s(drivePath, tipbase::MAXPATH, pPath, 3);
	if (GetDiskFreeSpaceEx(drivePath, (PULARGE_INTEGER)&FreeByteAvailableToCaller, NULL, NULL))
	{
		// Megabyte ȯ
		const float kilo = 1024.0;
		float freeSpaceMB = FreeByteAvailableToCaller / (kilo*kilo);
		SetProperty(_T("common.freevolume"), freeSpaceMB);

		// ġ  Ȯ
		if( GetNeedCapacity() > freeSpaceMB )
		{
			SetProperty(_T("volcolor"), L"#e3534a");
		}
		else
		{
			SetProperty(_T("volcolor"), L"#666666");
		}
	}
	else
	{
		SetProperty(_T("common.freevolume"), 0);
	}
}

void UpdaterDocument::SetServicesVolume( float volume )
{
	SetProperty(_T("common.volume_sum"), volume);
	
	// ġ  Ȯ
	if( volume > GetFreeCapacity() )
	{
		SetProperty(_T("volcolor"), L"#e3534a");
	}
	else
	{
		SetProperty(_T("volcolor"), L"#666666");
	}
}

void UpdaterDocument::SetListviewSelectionCount( int count )
{
	this->SetProperty(_T("listview_selectioncount"), count);
}


void UpdaterDocument::RecalcVolume( float volume )
{
	std::vector<TipServiceData*>::iterator it = m_ServiceList.begin();
	TipServiceData* installerServiceData = NULL;
	for (; it != m_ServiceList.end(); it++)
	{
		installerServiceData = dynamic_cast<TipServiceData*>((*it));
		if (installerServiceData->GetInstallSelect())
		{
			// ̼ġÿ 뷮 Ѵ.
			if (installerServiceData->GetInstallInfo() == InstallInfo::NOTINSTALLED)
			{
				volume += installerServiceData->GetCapacity();
			}
		}
	}
	SetServicesVolume(volume);
}

BOOL UpdaterDocument::IsInstallProcessFinish()
{
	BOOL bFinish = FALSE;
	TipServiceData* service;
	for (int i = 0; i < GetNumOfToBeInstall(); i++)
	{
		service = GetToBeInstall(i);
		if (service)
		{
			InstallStatus::type status = service->GetInstallStatus();
			if (!(status & InstallStatus::FINISH_SET))
			{
				bFinish = FALSE;
				break;
			}
			bFinish = TRUE;
		}
	}
	return bFinish;	
}

void UpdaterDocument::ConfirmInstallStatus(bool done)
{
	SetProperty(L"installview_isinstallcomplete", CComVariant(done));
}

void UpdaterDocument::CheckInstallFailed()
{
	// ġϷ  г ϳl Row ΰ  . 
	// ϳ ROW ΰ  ġ  ǥҼ Ƿ 
	// гο ʿ   Property λ ߰Ѵ.
	//  a_  b_ ٴ´.   ̸  ,   ε ߰Ǹ
	//  ߰ ־ Ѵ.
	int count = GetNumOfToBeInstall();

	IDataContext* pFinishTwoAppPanelDataContext = GetChild(L"installlistfinish");

	ATLASSERT( pFinishTwoAppPanelDataContext );
	bool bfailed = false;
	if ( pFinishTwoAppPanelDataContext )
	{
		// ٽ Ѵ.
		pFinishTwoAppPanelDataContext->RemoveAll();

		for (int i = 0; i < count; i ++)
		{
			SimpleMapDataContext* map = new SimpleMapDataContext();

			// ù° г ʱȭ
			TipServiceData* service = GetToBeInstall(i++);			
			map->SetProperty(_T("a_name"), CComVariant(service->GetName()));
			bool iserror = ( service->GetInstallStatus() == InstallStatus::FAIL 
				|| service->GetInstallStatus() == InstallStatus::CANCEL);
			if(iserror)
			{
				bfailed = true;
			}
			map->SetProperty(_T("a_partner"), service->IsPartner()? L"true" : L"false");
			map->SetProperty(_T("a_iserror"), CComVariant(iserror));
			map->SetProperty(_T("a_helperobject"), CComVariant(PtrToUint(service)));

			// ι° г ʱȭ
			service = GetToBeInstall(i);	
			if(service)
			{
				map->SetProperty(_T("b_name"), CComVariant(service->GetName()));
				iserror = ( service->GetInstallStatus() == InstallStatus::FAIL
					|| service->GetInstallStatus() == InstallStatus::CANCEL);
				if(iserror)
				{
					bfailed = true;
				}
				map->SetProperty(_T("b_partner"), service->IsPartner()? L"true" : L"false");
				map->SetProperty(_T("b_iserror"), CComVariant(iserror));
				map->SetProperty(_T("b_helperobject"), CComVariant(PtrToUint(service)));
				map->SetProperty(_T("bexist"), CComVariant(true));
			}
			else
			{
				map->SetProperty(_T("b_name"), CComVariant(_T("")));
				map->SetProperty(_T("b_iserror"), CComVariant(false));
				map->SetProperty(_T("b_partner"), L"false");
				map->SetProperty(_T("b_helperobject"), CComVariant(PtrToUint(NULL)));
				map->SetProperty(_T("bexist"), CComVariant(false));
			}

			pFinishTwoAppPanelDataContext->Add(NULL, map);
		}
	}

	SetProperty(L"installfailedexists", CComVariant(bfailed));
}

BOOL UpdaterDocument::IsUpdatable()
{
	CComVariant v = GetProperty(L"listview_isupdatable");
	if( v.vt == VT_BSTR )
	{
		return CString(v.bstrVal) == L"true";
	}

	return FALSE;
}

BOOL UpdaterDocument::IsToolbarExistInInstallList()
{
	BOOL toolbarExist = FALSE;
	int count = GetNumOfToBeInstall();
	TipServiceData* service = NULL;
	for (int i = 0; i < count; i++)
	{
		service = GetToBeInstall(i);
		if ( service->GetID().CompareNoCase( _T("39B419B3-D960-46b2-97CB-82F5BD61799B") ) == 0 )
		{
			toolbarExist = TRUE;
		}
	}
	return toolbarExist;
}


UpdaterXMLValidater::UpdaterXMLValidater()
{
	m_TypeDefiner.SetAt(_T("startpage.url"), UrlType);
	m_TypeDefiner.SetAt(_T("defaultsearch.clsid"), GuidType);
	m_TypeDefiner.SetAt(_T("defaultsearch.url"), UrlType);
	m_TypeDefiner.SetAt(_T("startpage.changed.copy"), StringType);
	m_TypeDefiner.SetAt(_T("defaultsearch.changed.copy"), StringType);
	m_TypeDefiner.SetAt(_T("guid"), GuidType);
	m_TypeDefiner.SetAt(_T("name"), StringType);
	m_TypeDefiner.SetAt(_T("introtext"), StringType);
	m_TypeDefiner.SetAt(_T("icon.url"), UrlType);
	m_TypeDefiner.SetAt(_T("alarmicon.url"), UrlType);
	m_TypeDefiner.SetAt(_T("exposureweight"), NumberType);
	m_TypeDefiner.SetAt(_T("introduce.url"), UrlType);
	m_TypeDefiner.SetAt(_T("setupfile.url"), UrlType);
	m_TypeDefiner.SetAt(_T("agentdllxml.url"), UrlType);
	m_TypeDefiner.SetAt(_T("update.silent"), BooleanType);
	m_TypeDefiner.SetAt(_T("update.copy"), StringType);
	m_TypeDefiner.SetAt(_T("update.showinfo"), BooleanType);
	m_TypeDefiner.SetAt(_T("update.infourl"), UrlType);
	m_TypeDefiner.SetAt(_T("processor"), StringType);
}
BOOL UpdaterXMLValidater::IsValid(LPCTSTR key, LPCTSTR data)
{
	BOOL bRetVal = TRUE;
	CString dataKey(key);
	CComVariant var;
	CAtlMap<CString, valueType>::CPair* pair = m_TypeDefiner.Lookup(dataKey);
	if (pair != NULL)
	{
		var = pair->m_value;
		switch(var.intVal)
		{
		case UrlType:
			bRetVal = IsValidURL(data);
			break;
		case GuidType:
			bRetVal = IsValidGUID(data);
			break;
		case NumberType:
			bRetVal = IsValidNumber(data);
			break;
		case BooleanType:
			bRetVal = IsValidBoolean(data);
			break;
		case StringType:
			bRetVal = IsValidString(data);
			break;
		}
	}

	return bRetVal;
}

};
